﻿using System;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;

namespace cs32_cs_webio_client
{
    public partial class Form1 : Form
    {
        Socket TCP_Client;

        byte[] receiveBuffer;
        string receiveString;

        private delegate void DelegateSub();
        
        public Form1()
        {
            InitializeComponent();
        }
        //-----------------------------------------------------------------------------


        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            this.closeConnection();
        }
        //-----------------------------------------------------------------------------


        private void Form1_Load(object sender, EventArgs e)
        {
            
            this.gb_io.Enabled = false;
            this.bt_disconnect.Enabled = false;
            this.sb_status.Text = "no connection";

        }
        //-----------------------------------------------------------------------------


        private void bt_connect_Click(object sender, EventArgs e)
        {

            if(this.ed_ip.Text != "" && this.ed_Port.Text != "")
            {
                try
                {
                    IPEndPoint ClientEP = new IPEndPoint(IPAddress.Parse(this.ed_ip.Text), int.Parse(this.ed_Port.Text));
                    TCP_Client = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);

                    TCP_Client.BeginConnect(ClientEP, new AsyncCallback(callback_connect), TCP_Client);
                }
                catch (Exception)
                {
                    closeConnection();
                    this.sb_status.Text = "Error while connecting!";
                }
            }else{
                this.sb_status.Text = "IP and Port needed!";
            }
        }
        //-----------------------------------------------------------------------------


        private void callback_connect(IAsyncResult ar)
        {

            try
            {
                TCP_Client.EndConnect(ar);

                this.sb_status.Text = "connected";

                this.Invoke(new DelegateSub(disconnectenable));

                receiveBuffer = new byte[512];
                TCP_Client.BeginReceive(receiveBuffer, 0, 512, SocketFlags.None, new AsyncCallback(callback_receive), TCP_Client);
            }
            catch (Exception)
            {
                closeConnection();
                this.sb_status.Text = "Error while connecting!";
            }

        }
        //-----------------------------------------------------------------------------


        private void callback_receive(IAsyncResult ar)
        {
             
            int receiveCount = 0;

            try
            {
                if (TCP_Client != null)
                {
                    if (TCP_Client.Connected)
                    {
                        receiveCount = TCP_Client.EndReceive(ar);
                        receiveString = Encoding.ASCII.GetString(receiveBuffer);
                        Array.Clear(receiveBuffer, 0, 512);
                        this.Invoke(new DelegateSub(evaluateResponse));
                        TCP_Client.BeginReceive(receiveBuffer, 0, 512, SocketFlags.None, new AsyncCallback(callback_receive), TCP_Client);
                    }
                }
            }
            catch (Exception)
            {
                this.sb_status.Text = "Error while receiving!";
            }

            if (receiveCount == 0)
            {
                closeConnection();
            }
        }
        //-----------------------------------------------------------------------------


        private void closeConnection()
        {

            try
            {
                if (TCP_Client != null)
                {
                   if(TCP_Client.Connected)
                   {
                        TCP_Client.Shutdown(SocketShutdown.Both);
                        TCP_Client.Close();
                        TCP_Client = null;
                   }
                }

                this.sb_status.Text = "no connection";

                this.Invoke(new DelegateSub(connectenable));
            }
            catch (Exception)
            {
                this.sb_status.Text = "Error while disconnecting!";
            }
        }
        //-----------------------------------------------------------------------------


        private void disconnectenable()
        {

            this.bt_disconnect.Enabled = true;
            this.bt_connect.Enabled = false;
            this.gb_io.Enabled = true;
        
        }
        //-----------------------------------------------------------------------------


        private void connectenable()
        {

            this.bt_disconnect.Enabled = false;
            this.bt_connect.Enabled = true;
            this.gb_io.Enabled = false;

        }
        //-----------------------------------------------------------------------------


        private void evaluateResponse()
        {

            char[] seperator = {';'};

            try
            {

                switch(receiveString.Substring(0, 1))
                {
                    case "i":
                        int inputValue = Convert.ToInt16(receiveString.Substring(6, 4), 16);

                        for(int port = 0; port<12; port++)
                        {
                            if ( ( (int)inputValue & (int)Math.Pow(2, port) ) > 0 )
                            {
                                ((CheckBox)this.Controls.Find("cb_input" + port, true)[0]).Checked = true;
                            }else{
                                ((CheckBox)this.Controls.Find("cb_input" + port, true)[0]).Checked =  false;
                            }
                        }

                        break;
                    case "o":
                        int outputValue = Convert.ToInt16(receiveString.Substring(7, 4), 16);

                        for(int port = 0; port<12; port++)
                        {
                            if ( ( (int)outputValue & (int)Math.Pow(2, port) ) > 0  )
                            {
                                ((CheckBox)this.Controls.Find("cb_output" + port, true)[0]).Checked = true;
                            }else{
                                ((CheckBox)this.Controls.Find("cb_output" + port, true)[0]).Checked = false;
                            }
                        }

                        break;
                    case "c":
                        if (receiveString.IndexOf(";") == 7)
                        {
                            string[] values = receiveString.Split(seperator);

                            for(int port = 0; port<12; port++)
                            {
                                ((TextBox)this.Controls.Find("ed_counter" + port, true)[0]).Text = values[port + 1];
                            }
                        }else{
                            int port = int.Parse(receiveString.Substring(7, receiveString.IndexOf(";") - 7));
                            int value = int.Parse(receiveString.Substring(receiveString.IndexOf(";") + 1));
                            ((TextBox)this.Controls.Find("ed_counter" + port, true)[0]).Text = value.ToString();
                        }
                        break;
                }
            }
            catch (Exception)
            {
                this.sb_status.Text = "Error while receiving";
            }

        }
        //-----------------------------------------------------------------------------


        private void sendCmd(string cmd)
        {

            try
            {
                byte[] cmdBytes;
                cmdBytes = Encoding.ASCII.GetBytes(cmd);
                TCP_Client.Send(cmdBytes, cmdBytes.Length, SocketFlags.None);
            }
            catch (Exception)
            {
                this.sb_status.Text = "Error while sending command!";
            }

        }
        //-----------------------------------------------------------------------------


        private void cb_output_Click(object sender, EventArgs e)
        {
            
            CheckBox Cur_cb = (CheckBox)sender;
            int CurOutputNo;

            if (Cur_cb.Name.Length == 10)
            {
                CurOutputNo = int.Parse(Cur_cb.Name.Substring(9, 1));
            }else{
                CurOutputNo = int.Parse(Cur_cb.Name.Substring(9, 2));
            }

            if (Cur_cb.Checked)
            {
                sendCmd("GET /outputaccess" + CurOutputNo.ToString() + "?PW=" + ed_Password.Text + "&State=ON&");
            }else{
                sendCmd("GET /outputaccess" + CurOutputNo.ToString() + "?PW=" + ed_Password.Text + "&State=OFF&");
            }

        }
        //-----------------------------------------------------------------------------


        private void bt_outputs_read_Click(object sender, EventArgs e)
        {
            
            sendCmd("GET /output?PW=" + ed_Password.Text + "&");

        }
        //-----------------------------------------------------------------------------


        private void bt_inputs_read_Click(object sender, EventArgs e)
        {

            sendCmd("GET /input?PW=" + ed_Password.Text + "&");

        }
        //-----------------------------------------------------------------------------


        private void bt_conter_readall_Click(object sender, EventArgs e)
        {

            sendCmd("GET /counter?PW=" + ed_Password.Text + "&");

        }
        //-----------------------------------------------------------------------------


        private void bt_counter_clearall_Click(object sender, EventArgs e)
        {

            sendCmd("GET /counterclear?PW=" + ed_Password.Text + "&");

        }
        //-----------------------------------------------------------------------------


        private void bt_counter_read_Click(object sender, EventArgs e)
        {
            
            Button Cur_bt = (Button)sender;
            int CurCounterNo;

            if (Cur_bt.Name.Length == 16)
            {
                CurCounterNo = int.Parse(Cur_bt.Name.Substring(15, 1));
            }else{
                CurCounterNo = int.Parse(Cur_bt.Name.Substring(15, 2));
            }

            sendCmd("GET /counter" + CurCounterNo.ToString() + "?PW=" + ed_Password.Text + "&");

        }
        //-----------------------------------------------------------------------------


        private void bt_counter_clear_Click(object sender, EventArgs e)
        {

            Button Cur_bt = (Button)sender;
            int CurCounterNo;

            if (Cur_bt.Name.Length == 17)
            {
                CurCounterNo = int.Parse(Cur_bt.Name.Substring(16, 1));
            }else{            
                CurCounterNo = int.Parse(Cur_bt.Name.Substring(16, 2));
            }

            sendCmd("GET /counterclear" + CurCounterNo.ToString() + "?PW=" + ed_Password.Text + "&");

        }
        //--------------------------------------------------------------------------------------------
        

        private void bt_disconnect_Click(object sender, EventArgs e)
        {

            this.closeConnection();

        }
        //--------------------------------------------------------------------------------------------


        private void ed_interval_TextChanged(object sender, EventArgs e)
        {

            this.Timer_polling.Interval = int.Parse(this.ed_interval.Text);

        }
        //--------------------------------------------------------------------------------------------


        private void Timer_polling_Tick(object sender, EventArgs e)
        {

            if (TCP_Client != null)
            {
                if (TCP_Client.Connected)
                {
                    if (cb_output_polling.Checked)
                    {
                        sendCmd("GET /output?PW=" + ed_Password.Text + "&");
                    }
                    if (cb_input_polling.Checked)
                    {
                        sendCmd("GET /input?PW=" + ed_Password.Text + "&");
                    }
                    if (cb_counter_polling.Checked)
                    {
                        sendCmd("GET /counter?PW=" + ed_Password.Text + "&");
                    }
                }
            }


        }
        //--------------------------------------------------------------------------------------------
    }
}
